
% function counter_config(this_block)
function counter_config(this_block)

  % -----------Added-------------
  % This code is the one that shows how to grab parameters (Width)
  %  from the masked counter block
  mybb = this_block.blockName;
  masked_counter = get_param(mybb,'Parent');
  
  % Work around: Create a structure of all the Parameter Names
  % and their evaluated values that are on the specified mask
  %
  % For MaskWSVariables See MATLAB Doc > Mask Parameters > Model and Block
  % Parameters > Mask Parameters > About Mask Parameters 
  % 
  maskParamNameValuePairs = get_param(masked_counter, 'MaskWSVariables');

  % now step through each MASK to get the name and the evaluated value
  count_width = -1;	%Initial values so to know if Mask is present.
  count_init = -1;
  count_max = -1;
  for i=1:length(maskParamNameValuePairs)
	  if (strcmpi(maskParamNameValuePairs(i).Name, 'count_width'))
		  count_width = maskParamNameValuePairs(i).Value;
	  end
	  if (strcmpi(maskParamNameValuePairs(i).Name, 'count_init'))
		  count_init = maskParamNameValuePairs(i).Value;
	  end
	  if (strcmpi(maskParamNameValuePairs(i).Name, 'count_max'))
		  count_max = maskParamNameValuePairs(i).Value;
	  end
  end
  % -----------End---------------


  % Revision History:
  %
  %   07-May-2010  (13:51 hours):
  %     Original code was machine generated by Xilinx's System Generator after parsing
  %     C:\douangp\12.2_to_be_delivered\CR535424\blackbox\counter.vhd
  %
  %

  this_block.setTopLevelLanguage('VHDL');

  this_block.setEntityName('counter');

  % System Generator has to assume that your entity  has a combinational feed through; 
  %   if it  doesn't, then comment out the following line:
  this_block.tagAsCombinational;

  this_block.addSimulinkInport('reset');
  this_block.addSimulinkInport('enable');

  this_block.addSimulinkOutport('count');

  % -----------------------------
  if (this_block.inputTypesKnown)
    % do input type checking, dynamic output type and generic setup in this code block.

    if (this_block.port('reset').width ~= 1);
      this_block.setError('Input data type for port "reset" must have width=1.');
    end

    this_block.port('reset').useHDLVector(false);

    if (this_block.port('enable').width ~= 1);
      this_block.setError('Input data type for port "enable" must have width=1.');
    end

    this_block.port('enable').useHDLVector(false);

  % (!) Port 'count' appeared to have dynamic type in the HDL
  % --- you must add an appropriate type setting for this port
  end  % if(inputTypesKnown)
  % -----------------------------

  % -----------Added-------------
  count = this_block.port('count');
  if (count_width ~= -1)
    count.setType(['UFix_' num2str(count_width) '_0']);
  end
  % -----------End---------------
  
  % -----------------------------
   if (this_block.inputRatesKnown)
     setup_as_single_rate(this_block,'clk','ce')
   end  % if(inputRatesKnown)
  % -----------------------------

    % (!) Set the inout port rate to be the same as the first input 
    %     rate. Change the following code if this is untrue.
    uniqueInputRates = unique(this_block.getInputRates);

  % (!) Custimize the following generic settings as appropriate. If any settings depend
  %      on input types, make the settings in the "inputTypesKnown" code block.
  %      The addGeneric function takes  3 parameters, generic name, type and constant value.
  %      Supported types are boolean, real, integer and string.
  %   this_block.addGeneric('COUNT_BIT_WIDTH','integer','4');
  %   this_block.addGeneric('COUNT_INITIAL','integer','0');
  %   this_block.addGeneric('COUNT_MAX','integer','15');
  % -----------Modified-------------
  this_block.addGeneric('COUNT_BIT_WIDTH','integer',num2str(count_width));
  this_block.addGeneric('COUNT_INITIAL','integer',num2str(count_init));
  this_block.addGeneric('COUNT_MAX','integer',num2str(count_max));
  % -----------End---------------

  % Add addtional source files as needed.
  %  |-------------
  %  | Add files in the order in which they should be compiled.
  %  | If two files "a.vhd" and "b.vhd" contain the entities
  %  | entity_a and entity_b, and entity_a contains a
  %  | component of type entity_b, the correct sequence of
  %  | addFile() calls would be:
  %  |    this_block.addFile('b.vhd');
  %  |    this_block.addFile('a.vhd');
  %  |-------------

  %    this_block.addFile('');
  %    this_block.addFile('');
  this_block.addFile('counter.vhd');

return;


% ------------------------------------------------------------

function setup_as_single_rate(block,clkname,cename) 
  inputRates = block.inputRates; 
  uniqueInputRates = unique(inputRates); 
  if (length(uniqueInputRates)==1 & uniqueInputRates(1)==Inf) 
    block.addError('The inputs to this block cannot all be constant.'); 
    return; 
  end 
  if (uniqueInputRates(end) == Inf) 
     hasConstantInput = true; 
     uniqueInputRates = uniqueInputRates(1:end-1); 
  end 
  if (length(uniqueInputRates) ~= 1) 
    block.addError('The inputs to this block must run at a single rate.'); 
    return; 
  end 
  theInputRate = uniqueInputRates(1); 
  for i = 1:block.numSimulinkOutports 
     block.outport(i).setRate(theInputRate); 
  end 
  block.addClkCEPair(clkname,cename,theInputRate); 
  return; 

% ------------------------------------------------------------
